# -*- coding: utf-8 -*-

class Documento:
    """
    Representa un documento de texto con un cuerpo, título, y clasificación.

    Atributos:
        - path (str): Ruta del fichero del que se ha parseado el documento.
        - id (str): Identificador único.
        - cuerpo (str)
        - titulo (str)
        - clasificacion (str)
    """
    def __init__(self):
        self.__path = ""
        self.__id = ""
        self.__cuerpo = ""
        self.__titulo = ""
        self.__clasificacion = ""

    def setPath(self,path):
        """
        Setter del atributo path.

        Argumentos:
            - path (str)
        """
        self.__path = path

    def setId(self,idf):
        """
        Setter del atributo id.

        Argumentos:
            - idf (str)
        """
        self.__id = idf

    def setCuerpo(self,cuerpo):
        """
        Setter del atributo cuerpo.

        Argumentos:
            - cuerpo (str)
        """
        self.__cuerpo = cuerpo

    def setTitulo(self,titulo):
        """
        Setter del atributo titulo.

        Argumentos:
            - titulo (str)
        """
        self.__titulo = titulo

    def setClasificacion(self,clasificacion):
        """
        Setter del atributo clasificacion.

        Argumentos:
            - clasificacion (str)
        """
        self.__clasificacion = clasificacion

    def getPath(self):
        """
        Getter del atributo path.

        Devuelve:
            - path (str)
        """
        return self.__path

    def getId(self):
        """
        Getter del atributo id.

        Devuelve:
            - id (str)
        """
        return self.__id

    def getCuerpo(self):
        """
        Getter del atributo cuerpo.

        Devuelve:
            - cuerpo (str)
        """
        return self.__cuerpo

    def getTitulo(self):
        """
        Getter del atributo titulo.

        Devuelve:
            - titulo (str)
        """
        return self.__titulo

    def getClasificacion(self):
        """
        Getter del atributo clasificacion.

        Devuelve:
            - clasificacion (str)
        """
        return self.__clasificacion


class Modelo:
    """
    Representa un modelo conteniendo todos los datos necesarios para recrear
    una red neuronal.

    Atributos:
        - id (str): Identificador único.
        - nombre (str)
        - red (str): Tipo de red.
        - pesos (dict:str->list:list:double): Representación de los pesos de la red, en un diccionarioque tiene como clave las capas que conecta y como valor una lista de listas con los pesos.
        - bias (dict:str->list:double): Representación de los términos bias de la red, en un diccionario con clave la capa donde está la neurona, y como valor la lista de bias.
        - clases (list:str): Lista de etiquetas que representan las posibles clases de la red.
        - numeroNeuronas (int): Número de neuronas en la capa intermedia de la red. El número de neuronas de salida se pueden calcular directamente de la lista de clases.
        - vocabulario (dict:str->int): Transformación 1-1 entre palabras y valores numéricos enteros necesario para transformar el texto en números introducibles en las redes.
        - maxNormalizado (double): Valor utilizado como max en la función de normalización (x-min)/(min+max) que se debe aplicar a los valores resultantes del vocabulario antes de introducirlos a la red.
        - minNormalizado (double): Valor utilizado como min en la función de normalización (x-min)/(min+max) que se debe aplicar a los valores resultantes del vocabulario antes de introducirlos a la red.
    """
    def __init__(self):
        self.__id = ""
        self.__nombre = ""
        self.__red = ""
        self.__pesos = None
        self.__bias = None
        self.__clases = None
        self.__numeroNeuronas = 0
        self.__vocabulario = None
        self.__maxNormalizado = 0.0
        self.__minNormalizado = 0.0

    def setId(self,identificador):
        """
        Setter del atributo id.

        Argumentos:
            - identificador (str)
        """
        self.__id = identificador

    def setNombre(self,nombre):
        """
        Setter del atributo nombre.

        Argumentos:
            - nombre (str)
        """
        self.__nombre = nombre

    def setTipoRed(self,tipoRed):
        """
        Setter del atributo red.

        Argumentos:
            - tipoRed (str)
        """
        self.__red = tipoRed.upper()

    def setPesos(self,pesos):
        """
        Setter del atributo pesos.

        Argumentos:
            - pesos (dict:str->list:list:double)
        """
        self.__pesos = pesos

    def setBias(self,bias):
        """
        Setter del atributo bias.

        Argumentos:
            - bias (dict:str->list:double)
        """
        self.__bias = bias

    def setClases(self,clases):
        """
        Setter del atributo clases.

        Argumentos:
            - clases (list:str)
        """
        self.__clases = clases

    def setNumeroNeuronasOculta(self,numeroNeuronas):
        """
        Setter del atributo numNeuronas.

        Argumentos:
            - numeroNeuronas (int)
        """
        self.__numeroNeuronas = numeroNeuronas

    def setVocabulario(self,voc):
        """
        Setter del atributo vocabulario.

        Argumentos:
        
            - voc (dict:str->int)
        """
        self.__vocabulario = voc

    def setMaximoNormalizado(self,m):
        """
        Setter del atributo maxNormalizado.

        Argumentos:
        
            - m (double)
        """
        self.__maxNormalizado = m

    def setMinimoNormalizado(self,m):
        """
        Setter del atributo minNormalizado.

        Argumentos:
        
            - m (double)
        """
        self.__minNormalizado = m

    def getId(self):
        """
        Getter del atributo id.

        Devuelve:
        
            - identificador (str)
        """
        return self.__id

    def getNombre(self):
        """
        Getter del atributo nombre.

        Devuelve:
        
            - nombre (str)
        """
        return self.__nombre

    def getTipoRed(self):
        """
        Getter del atributo red.

        Devuelve:
        
            - tipoRed (str)
        """
        return self.__red

    def getPesos(self):
        """
        Getter del atributo pesos.

        Devuelve:
        
            - pesos (dict:str->list:list:double)
        """
        return self.__pesos

    def getBias(self):
        """
        Getter del atributo bias.

        Devuelve:
        
            - bias (dict:str->list:double)
        """
        return self.__bias

    def getClases(self):
        """
        Getter del atributo clases.

        Devuelve:
        
            - clases (list:str)
        """
        return self.__clases

    def getNumeroNeuronasOculta(self):
        """
        Getter del atributo numNeuronas.

        Devuelve:
        
            - numeroNeuronas (int)
        """
        return self.__numeroNeuronas

    def getVocabulario(self):
        """
        Getter del atributo vocabulario.

        Devuelve:
        
            - voc (dict:str->int)
        """
        return self.__vocabulario

    def getMaximoNormalizado(self):
        """
        Getter del atributo maxNormalizado.

        Devuelve:
        
            - m (double)
        """
        return self.__maxNormalizado

    def getMinimoNormalizado(self):
        """
        Getter del atributo minNormalizado.

        Devuelve:
        
            - m (double)
        """
        return self.__minNormalizado


class ExperimentoEntrenamiento:
    """
    Representa un experimento de entrenamiento, conteniendo todos los
    datos necesarios para entrenar una red neuronal.

    Atributos:
        - modeloGenerado (modelo.Modelo): Modelo que se produce tras entrenar la red con los parámetros indicados.
        - conjuntoEntrenamiento (list:modelo.Documento)
        - conjuntoValidacion (list:modelo.Documento)
        - factorAprendizaje (double): Este factor es el que multiplica a la variación de pesos en la fase de retropropagación del error.
        - factorInercia (double):  Este factor es el que multiplica la variación de pesos obtenida de la instancia anterior a la actual en la fase de retropropagación del error.
        - numeroEpocas (int): Número de iteraciones por todo el conjunto de entrenamiento al que se someterá a la red.
        - metodoValidacion (str): Método utilizado para dividir el conjunto de documentos en conjunto de entrenamiento y conjunto de validación.
        - parametroValidacion (double): Parámetro utilizado para el método de validación seleccionado.
        - semillaAleatoria (int): Semilla aleatoria a utilizar para cualquier generador de números aleatorios utilizado en el experimento.
        - stopWordsExtra (list:str): Lista de palabras a eliminar de los textos.
        - ordenVocabulario (str): Indica el orden en el que se asignan los valores a las palabras del vocabulario.
        - idioma (str): Idioma de los textos.
    """
    def __init__(self):
        self.__modeloGenerado = None
        self.__conjuntoEntrenamiento = []
        self.__conjuntoValidacion = []
        self.__factorAprendizaje = 0.0
        self.__factorInercia = 0.0
        self.__numeroEpocas = 0
        self.__metodoValidacion = ""
        self.__parametroValidacion = 0.0
        self.__semillaAleatoria = None
        self.__stopWordsExtra = []
        self.__ordenVocabulario = ""
        self.__idioma = ""

    def setModelo(self,modelo):
        """
        Setter del atributo modeloGenerado.

        Argumentos:
        
            - modelo (modelo.Modelo)
        """
        self.__modeloGenerado = modelo

    def setConjuntoEntrenamiento(self,c):
        """
        Setter del atributo conjuntoEntrenamiento.

        Argumentos:
        
            - c (list:modelo.Documento)
        """
        self.__conjuntoEntrenamiento = c

    def setConjuntoValidacion(self,c):
        """
        Setter del atributo conjuntoValidacion.

        Argumentos:
        
            - c (list:modelo.Documento)
        """
        self.__conjuntoValidacion = c

    def setFactorAprendizaje(self,f):
        """
        Setter del atributo factorAprendizaje.

        Argumentos:
        
            - f (double)
        """
        self.__factorAprendizaje = f

    def setFactorInercia(self,f):
        """
        Setter del atributo factorInercia.

        Argumentos:
        
            - f (double)
        """
        self.__factorInercia = f

    def setNumeroEpocas(self,n):
        """
        Setter del atributo numeroEpocas.

        Argumentos:
        
            - n (int)
        """
        self.__numeroEpocas = n

    def setMetodoValidacion(self,metodo):
        """
        Setter del atributo metodoValidacion.

        Argumentos:
        
            - metodo (str)
        """
        self.__metodoValidacion = metodo.upper()

    def setParametroValidacion(self,par):
        """
        Setter del atributo parametroValidacion.

        Argumentos:
        
            - par (double)
        """
        self.__parametroValidacion = par

    def setSemillaAleatoria(self,seed):
        """
        Setter del atributo semillaAleatoria.

        Argumentos:
        
            - seed (int)
        """
        self.__semillaAleatoria = seed

    def setStopWordsExtra(self,sw):
        """
        Setter del atributo stopWordsExtra.

        Argumentos:
        
            - sw (list:str)
        """
        self.__stopWordsExtra = sw

    def setOrdenVocabulario(self,orden):
        """
        Setter del atributo ordenVocabulario.

        Argumentos:
            - orden (str): orden en el que se asignan los valores a las palabras del vocabulario.
        """
        self.__ordenVocabulario = orden.upper()
        
    def setIdioma(self,idioma):
        """
        Setter del atributo idioma.
        
        Argumentos:
            - idioma (str): idioma de los textos
        """
        self.__idioma = idioma.lower()
        
    def getModelo(self):
        """
        Getter del atributo modeloGenerado.

        Devuelve:
        
            - modelo (modelo.Modelo)
        """
        return self.__modeloGenerado

    def getConjuntoEntrenamiento(self):
        """
        Getter del atributo conjuntoEntrenamiento.

        Devuelve:
        
            - c (list:modelo.Documento)
        """
        return self.__conjuntoEntrenamiento

    def getConjuntoValidacion(self):
        """
        Getter del atributo conjuntoValidacion.

        Devuelve:
        
            - c (list:modelo.Documento)
        """
        return self.__conjuntoValidacion

    def getFactorAprendizaje(self):
        """
        Getter del atributo factorAprendizaje.

        Devuelve:
        
            - f (double)
        """
        return self.__factorAprendizaje

    def getFactorInercia(self):
        """
        Getter del atributo factorInercia.

        Devuelve:
        
            - f (double)
        """
        return self.__factorInercia

    def getNumeroEpocas(self):
        """
        Getter del atributo numeroEpocas.

        Devuelve:
        
            - numeroEpocas (int)
        """
        return self.__numeroEpocas

    def getMetodoValidacion(self):
        """
        Getter del atributo metodoValidacion.

        Devuelve:
        
            - metodo (str)
        """
        return self.__metodoValidacion

    def getParametroValidacion(self):
        """
        Getter del atributo parametroValidacion.

        Devuelve:
        
            - par (double)
        """
        return self.__parametroValidacion

    def getSemillaAleatoria(self):
        """
        Getter del atributo semillaAleatoria.

        Devuelve:
        
            - semillaAleatoria (int)
        """
        return self.__semillaAleatoria

    def getStopWordsExtra(self):
        """
        Getter del atributo stopWordsExtra.

        Devuelve:
        
            - stopWordsExtra (list:str)
        """
        return self.__stopWordsExtra


    def getOrdenVocabulario(self):
        """
        Gtter del atributo ordenVocabulario.

        Devuelve:
            - orden (str): orden en el que se asignan los valores a las palabras del vocabulario.
        """
        return self.__ordenVocabulario
    
    def getIdioma(self):
        """
        Getter del atributo idioma.
        
        Devuelve:
            - idioma (str): idioma de los textos
        """
        return self.__idioma
    

class EstadoEntrenamiento:
    """
    Representa el estado de un experimento de entrenamiento.

    Atributos:
        - tamEntrenamiento (int): Tamaño del conjunto de entrenamiento.
        - tamValidacion (int): Tamaño del conjunto de validación.
        - clases (list:str): Lista de clases entrenadas.
        - proporcionClases (list:double): Lista de proporciones de cada clase en el conjunto de entrenamiento.
        - tasasEntrenamiento (list:double): Lista de las tasas de acierto en el conjunto de entrenamiento por iteracion.
        - tasasValidacion (list:double): Lista de las tasas de validacion en el conjunto de entrenamiento por iteracion.
         - matrizConfusion (dict): Diccionario con las claves 'matriz', 'filas', y 'columnas', conteniendo:
                 - matriz (list:list:int): Matriz de confusión.
                 - filas (list:str): Lista de etiquetas para las filas de la matriz.
                 - columnas (list:str): Lista de etiquetas para las columnas de la matriz.
        - epocasARealizar (int): Número de epocas totales a realizar.
        - epocasRealizadas (int): Número de epocas realizadas.
        - experimentosARealizar (int): Número de experimentos totales a realizar.
        - experimentosRealizados (int): Número de experimentos realizados.
    """
    def __init__(self):
        self.tamEntrenamiento = 0
        self.tamValidacion = 0
        self.clases = list()
        self.proporcionClases = list()
        self.tasasEntrenamiento = list()
        self.tasasValidacion = list()
        self.matrizConfusion = {'matriz':[[]],'filas':[],'columnas':[]}
        self.epocasARealizar = 0
        self.epocasRealizadas = 0
        self.experimentosARealizar = 1
        self.experimentosRealizados = 0

    def setTamEntrenamiento(self,t):
        """
        Setter del atributo tamEntrenamiento.

        Argumentos:
            - t (int): tamaño del conjunto de entrenamiento
        """
        self.tamEntrenamiento = t

    def setTamValidacion(self,v):
        """
        Setter del atributo tamValidacion.

        Argumentos:
            - v (int): tamaño del conjunto de validacion
        """
        self.tamValidacion = v

    def setClases(self,c):
        """
        Setter del atributo clases.

        Argumentos:
            - c (list:str): lista de etiquetas de clases
        """
        self.clases = c

    def setProporcionClases(self,p):
        """
        Setter del atributo proporcion.

        Argumentos:
            - p (list:double): lista de las proporciones de cada clase en el conjunto de entrenamiento. Debe estar en el mismo orden que el atributo `clases`
        """
        self.proporciones = p

    def setTasasEntrenamiento(self,tasas):
        """
        Setter del atributo tasasEntrenamiento.

        Argumentos:
            - tasas (list:double): lista de la tasa de acierto en el conjunto de entrenamiento en cada iteración.
        """
        self.tasasEntrenamiento = tasas


    def setTasasValidacion(self,tasas):
        """
        Setter del atributo tasaValidacion.

        Argumentos:
            - tasas (list:double): lista de la tasa de acierto en el conjunto de validacion en cada iteración.
        """
        self.tasasValidacion = tasas

    def setMatrizConfusion(self,matriz):
        """
        Setter del atributo matrizConfusion.

        Argumentos:
            - matriz (dict): Diccionario con las claves 'matriz', 'filas', y 'columnas', conteniendo:
                 - matriz (list:list:int): Matriz de confusión.
                 - filas (list:str): Lista de etiquetas para las filas de la matriz.
                 - columnas (list:str): Lista de etiquetas para las columnas de la matriz.
        """
        self.matrizConfusion = matriz

    def setEpocasARealizar(self,it):
        """
        Setter del atributo epocasARealizar.

        Argumentos:
            - it (int)
        """
        self.epocasARealizar = it

    def setEpocasRealizadas(self,it):
        """
        Setter del atributo epocasRealizadas.

        Argumentos:
            - it (int)
        """
        self.epocasRealizadas = it

    def setExperimentosARealizar(self,ex):
        """
        Setter del atributo experimentosARealizar.

        Argumentos:
            - ex (int)
        """
        self.experimentosARealizar = ex

    def setExperimentosRealizados(self,ex):
        """
        Setter del atributo experimentosRealizados.

        Argumentos:
            - ex (int)
        """
        self.experimentosRealizados = ex

    def getTamEntrenamiento(self):
        """
        Getter del atributo tamEntrenamiento.

        Devuelve:
            - tamEntrenamiento (int): tamaño del conjunto de entrenamiento
        """
        return self.tamEntrenamiento

    def getTamValidacion(self):
        """
        Getter del atributo tamValidacion.

        Devuelve:
            - tamValidacion (int): tamaño del conjunto de validacion
        """
        return self.tamValidacion

    def getClases(self):
        """
        Getter del atributo clases.

        Devuelve:
            - clases (list:str): lista de etiquetas de clases
        """
        return self.clases

    def getProporcionClases(self):
        """
        Getter del atributo proporcion.

        Devuelve:
            - proporciones (list:double): lista de las proporciones de cada clase en el conjunto de entrenamiento.
        """
        return self.proporciones

    def getTasasEntrenamiento(self):
        """
        Getter del atributo tasasEntrenamiento.

        Devuelve:
            - tasasEntrenamiento (list:double): lista de la tasa de acierto en el conjunto de entrenamiento en cada iteración.
        """
        return self.tasasEntrenamiento


    def getTasasValidacion(self):
        """
        Getter del atributo tasaValidacion.

        Devuelve:
            - tasasValidacion (list:double): lista de la tasa de acierto en el conjunto de validacion en cada iteración.
        """
        return self.tasasValidacion

    def getMatrizConfusion(self):
        """
        Getter del atributo matrizConfusion.

        Devuelve:
            - matrizConfusion (dict): Diccionario con las claves 'matriz', 'filas', y 'columnas', conteniendo:
                - matriz (list:list:int): Matriz de confusión.
                - filas (list:str): Lista de etiquetas para las filas de la matriz.
                - columnas (list:str): Lista de etiquetas para las columnas de la matriz.
        """
        return self.matrizConfusion

    def getEpocasARealizar(self):
        """
        Getter del atributo epocasARealizar.

        Devuelve:
            - epocasARealizar (int)
        """
        return self.epocasARealizar

    def getEpocasRealizadas(self):
        """
        Getter del atributo epocasRealizadas.

        Devuelve:
            - epocasRealizadas (int)
        """
        return self.epocasRealizadas

    def getExperimentosARealizar(self):
        """
        Getter del atributo experimentosARealizar.

        Devuelve:
            - experimentosARealizar (int)
        """
        return self.experimentosARealizar

    def getExperimentosRealizados(self):
        """
        Getter del atributo experimentosRealizados.

        Devuelve:
            - experimentosRealizados (int)
        """
        return self.experimentosRealizados
    
class ExperimentoClasificacion:
    """
    Representa un experimento de clasificacion, conteniendo todos los
    datos necesarios para obtener las clases de una serie de documentos
    a partir de una red ya entrenada.

    Atributos:
        - modelo (modelo.Modelo)
        - conjuntoClasificacion (list:modelo.Documento)
    """
    def __init__(self):
        self.__modelo = None
        self.__conjuntoClasificacion = None

    def setModelo(self,modelo):
        """
        Setter del atributo modelo

        Argumentos:
        
            - modelo (modelo.Modelo)
        """
        self.__modelo = modelo

    def setConjuntoClasificacion(self,c):
        """
        Setter del atributo conjuntoClasificacion

        Argumentos:
        
            - conjuntoClasificacion (list:modelo.Fichero)
        """
        self.__conjuntoClasificacion = c

    def getModelo(self):
        """
        Getter del atributo modelo

        Devuelve:
        
            - modelo (modelo.Modelo)
        """
        return self.__modelo

    def getConjuntoClasificacion(self):
        """
        Getter del atributo conjuntoClasificacion

        Devuelve:
        
            - conjuntoClasificacion (list:modelo.Fichero)
        """
        return self.__conjuntoClasificacion
