# -*- coding: utf-8 -*-

class Normalizador:
    """
    Implementa la normalización de vectores numéricos en el rango (-inf,inf) a un rango definido R [minR,maxR].

    Atributos:
        - max (float): Valor máximo
        - min (float): Valor mínimo

    Argumentos:
        - minRango (float): Minimo de R. 0.1 por defecto
        - maxRango (float): Máximo de R. 0.9 por defecto
    """
    def __init__(self,minRango=0.1,maxRango=0.9):
        self.max = 0
        self.min = 0
        self.minRango = minRango
        self.maxRango = maxRango
        self.norm = None

    def fit_transform(self,X):
        """
        Calcula la función necesaria para normalizar X al rango R y devuelve X normalizada

        Argumentos:

            - X: (list:list:int): valores a calcular la función y normalizar

        Devuelve:

            - X_norm: (list:list:float): resultado de convertir X al rango R
        """
        flat_X = [x for l in X for x in l]
        self.max = max(flat_X)
        self.min = min(flat_X)

        self.norm = lambda x: (((x-self.min)/(self.min+self.max))*(self.maxRango-self.minRango))+self.minRango

        for i,el in enumerate(X):
            X[i] = [self.norm(item) for item in el]

        return X

    def transform(self,X):
        """
        Devuelve X normalizada al rango definido anteriormente con fit/fit_transform/setMin y setMax

        Argumentos:

            - X (list:list:int): valores a normalizar

        Devuelve:

            - X_norm (list:list:float): resultado de convertir X al rango R
        """
        self.norm = lambda x: (((x-self.min)/(self.min+self.max))*(self.maxRango-self.minRango))+self.minRango
        for i,el in enumerate(X):
            X[i] = [self.norm(item) for item in el]

        return X

    def setMin(self,mi):
        """
        Permite definir `min`, tal que la función de normalización convierta `min` a `minR`

        Argumentos:

            - mi (float): `min`
        """
        self.min = mi

    def setMax(self,ma):
        """
        Permite definir `max`, tal que la función de normalización convierta `max` a `maxR`

        Argumentos:

            - ma (float): `max`
        """
        self.max = ma

    def getMin(self):
        """
        Devuelve `min`, tal que la función de normalización actual convierte `min` a `minR`

        Devuelve:

            - min (float).
        """
        return self.min

    def getMax(self):
        """
        Devuelve `max`, tal que la función de normalización actual convierte `max` a `maxR`

        Devuelve:

            - max: (float)
        """
        return self.max
