# -*- coding: utf-8 -*-

from bs4 import BeautifulSoup
import re
from Modelo.modelo import Documento

class SGMLParser:
    """
    Clase con un solo método estático `parse`.
    """
    @staticmethod
    def parse(path):
        """
        Parsea el fichero en ruta `path` con la estructura xml con uno o más DOCs con al menos las siguientes etiquetas:
        <DOC>
            <TITLE></TITLE>
            <CATEGORY></CATEGORY>
            <TEXT></TEXT>
        </DOC>

        Argumentos:
            - path (str): ruta del fichero a parsear

        Devuelve:
            - output (list:modelo.Documento): lista de documentos resultado del parseo

        Excepciones:
            -
        """
        try:
            with open(path,'rb') as fp:
                soup = BeautifulSoup(fp,'html.parser')

            output = list()

            files = soup.findAll('doc')
            if(len(files)>0): #se encuentra la etiqueta 'doc'
                for f in files:
                    foo = Documento()
                    titulo = f.find('title')
                    if(titulo!=None):
                        titulo = titulo.text
                        #limpiamos los espacios y tabuladores del sitio, y cambiamos comillas simples por comillas dobles.
                        titulo = re.sub('\n+',' ',titulo)
                        titulo = re.sub("\r+",'',titulo)#por compatibiliad en unix, eliminamos \r, que no significa nada en este OS
                        titulo = re.sub('\t+',' ',titulo)

                        foo.setTitulo(titulo)

                    clasificacion = f.find('category')
                    if(clasificacion!=None):
                        clasificacion = clasificacion.text
                        #lo mismo con clasificacion
                        clasificacion = re.sub('\n+',' ',clasificacion)
                        clasificacion = re.sub("\r+",'',clasificacion)#por compatibiliad en unix, eliminamos \r, que no significa nada en este OS
                        clasificacion = re.sub('\t+',' ',clasificacion)
                        foo.setClasificacion(clasificacion)

                    cuerpo = f.find('text')
                    if(cuerpo==None): #no hay cuerpo. No permitimos crear ficheros sin cuerpo
                        raise AttributeError("Fichero mal formado.")
                    else:
                        cuerpo = cuerpo.text
                        #en el cuerpo no eliminamos espacios ni tabuladores
                        cuerpo = re.sub("\r+",'',cuerpo)#por compatibiliad en unix, eliminamos \r, que no significa nada en este OS

                        foo.setCuerpo(cuerpo)

                    foo.setPath(path)

                    output.append(foo)
            else: #no se ha encontrado doc
                raise AttributeError("Fichero mal formado.")
        except:
            raise AttributeError("Fichero mal formado.")

        return output
