# -*- coding: utf-8 -*-

"""
Este modulo contiene funciones auxiliares utilizadas por las redes.
"""

from numpy import exp, clip
from numpy.random import random_sample

def sigmoide(x):
   """
   Calcula la función sigmoide (1/(1+exp(-x))
   
   Argumentos:
       - x (array:float): Variable independiente de la función.
   
   Devuelve:
       - F(x) (array:float): Resultado de aplicar la función sigmoide a x.
   """
   #para evitar overflow debemos limitar x
   x = clip(x,-709,709)
   return 1/(1+exp(-x))

def dsigmoide(f):
    """
    Calcula la derivada de la función sigmoide f(1-f)
    
    Argumentos:
        - f (array: float): Función sigmoide.
    
    Devuelve:
        - f'(array: float): Derivada de la función sigmoide.
    """
    return f*(1-f)

def inicializaPeso(forma,a=-0.5,b=0.5):
    """
    Devuelve una matriz de valores aleatorios de la distribución uniforme continua en el intervalo [a,b).
    
    Argumentos:
        - forma (tuple: int): Forma que tendrá la matriz resultante. Una tupla de n elementos da una matriz de n dimensiones con forma[i] como número de elementos en la dimensión i.
        - a (float): Mínimo del rango en el que estará el peso.
        - b (float): Máximo del rango en el que estará el peso.
    
    Devuelve:
        - peso (array: float): Matriz de valores en el intervalo [a,b) con la forma indicada.
    """
    return (a-b)*random_sample(forma)-a